function [s, S, X, f, t, A, D]=duet(x, wa, ws, dsplit, hop, L, sr);
%function [s, S, X, f, t, A, D]=duet(x, wa, ws, dsplit, hop, L, sr);
%
% The source separation algorithm by Rickard et al.
%
% inputs:
%  x      : input signals (column vectors)
%  wa     : analysis window
%  ws     : synthesis window
%  hop    : hop size (default length(wa)/2 )
%  L      : FFT size (default length(wa) )
%  sr     : sampling rate (default 44100)
%  dsplit : clustering boundaries in samples (default 0)
%           e.g. [-2 2] gives 3 sources, containing the bins with
%               delays in the range (-inf , -2)
%               delays in the range (-2 , 2) 
%               delays in the range (2 , inf)
%
% outputs:
%  s : separated signals
%  S : spectrograms of separated signals
%  X : spectrograms of input signals
%  f : frequency axis
%  t : time axis
%  A : Relative amplitude of spectrograms X
%  D : Delay (relative phase) of spectrograms X
%
%

if nargin<7, sr=44100; end
if nargin<6, L=length(wa); end
if nargin<5, hop=length(wa)/2; end
if nargin<4, dsplit=0; end

% make sure input signals are in column vectors
if size(x,1)<size(x,2), x=x.'; end

[X,f,t]=stft(x, wa, hop, L, 0, sr);

A=abs(X{2}./X{1});
[F,T]=size(A);
D=imag(log(X{1}./X{2}))./([1 1:F-1]'/(F-1)*pi*ones(1,T));
% D= (angle(X{1})-angle(X{2})) ./([........... <== equivalent to above

% CLUSTERING ALGORITHM, GENERATING BINARY MASKS
nsources=length(dsplit)+1;  
M=cell(1,nsources);
M{1}=(D<=dsplit(1));  
for k=2:nsources-1,
  M{k}=(D>dsplit(k-1) & D<=dsplit(k));  
end
M{end}=(D>dsplit(end)); 

% SEPARATE THE SOURCES (MULTIPLY STFT WITH BINARY MASKS)
S=cell(1,nsources);
for s=1:nsources,
  S{s}=M{s}.*X{1};
end
s=istft(S, ws, hop, L, 0);




%PLOTTING DEMO
FMAX=sr/2;
while FMAX,
  C=5;
  FMAXbin=round(FMAX/sr*L);
  Yax=[-1.1 1.1]*max([max(abs(x)) max(abs(s))]);
  Fax=[0 FMAX];
  Tax=[0 max(t)];
  nsensors=size(x,2);
  R=2;
  for k=1:R,
    subplot(R,C,(k-1)*C+1);
    plot([0:length(x)-1]/44100,x(:,k));
    ylabel(['#' num2str(k)]);
    axis([Tax Yax]);
    if k==1, title('Mixture signals'); end
    if k==R, xlabel('Time [s]'); end
  end
  for k=1:R,
    subplot(R,C,(k-1)*C+2);
    imagesc(t,f,20*log10(abs(X{k})));
    axis xy;
    axis([Tax Fax]);
    if k==1, title('Mixture signals'); end
    if k==R, xlabel('Time [s]'); end
  end
  
  DMAX=5;
  subplot(2,C,3);
  DD=D(1:FMAXbin,:);
  hist(DD(:),[-DMAX:DMAX/50:DMAX]);
  ax=axis; axis([-DMAX DMAX ax(3:4)]);
  xlabel('Delay [samples]');
  title('Histograms');
  
  AMAX=10;
  subplot(2,C,3+C);
  AA=20*log10(A(1:FMAXbin,:));
  hist(AA(:),[-AMAX:AMAX/50:AMAX]);
  xlabel('Relative amplitude [dB]');
  ax=axis; axis([-DMAX DMAX ax(3:4)]);
  
  R=nsources;
  for k=1:R,
    subplot(R,C,(k-1)*C+4);
    imagesc(t,f,20*log10(abs(S{k})));
    axis xy;
    axis([Tax Fax]);
    ylabel(['#' num2str(k)]);
    if k==1, title('Separated signals'); end
    if k==R, xlabel('Time [s]'); end
  end
  for k=1:R,
    subplot(R,C,(k-1)*C+5);
    plot([0:length(s)-1]/44100,s(:,k));
    axis([Tax Yax]);
    if k==1, title('Separated signals'); end
    if k==R, xlabel('Time [s]'); end
  end
  FMAX=input('Type maximum frequency (0=quit) : ');
end